﻿/*
This source file is part of the RGBZ Video Camera project published in the paper

  Coherent Spatiotemporal Filtering, Upsampling and Rendering of RGBZ Videos
  Christian Richardt, Carsten Stoll, Neil A. Dodgson, Hans-Peter Seidel, Christian Theobalt
  Computer Graphics Forum (Proceedings of Eurographics), 31(2), May 2012
  <http://www.mpi-inf.mpg.de/resources/rgbz-camera/>

If you use our code, please cite our paper. Thanks.

Copyright (c) 2012 Christian Richardt <christian@richardt.name>


Code licence: GPL v3
----------------------------------------------------------------------------------------------------
This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, see <http://www.gnu.org/licenses/>.

$Id: StereoWarper.vert 537 2012-04-04 15:35:42Z cr333 $
*/
#version 120
#extension GL_EXT_gpu_shader4 : enable

// coordinates of vertices
uniform sampler2D coordinates;

// the vertex position, in viewing and texture coordinates
varying vec3 v3;
varying vec2 coords;

// forward declaration of mesh vertex shader functions (from Mesh.vert)
vec3 getVertex(sampler2D depthmap, vec2 position);

void main(void)  
{
	// vertex position and normal, in world coordinates
	vec3 vertex = getVertex(coordinates, gl_Vertex.xy);
	coords = gl_Vertex.xy; // vertices located at texels

	// transform SR4000 coordinate system (x left, y up, z into the scene)
	// to calibrated camera coordinate system (x right, y down, z into the scene)
	vec4 v = vec4(-1, -1, 1, 1) * vec4(vertex, 1);

	// vertex in eye space
	vec4 v4 = gl_ModelViewMatrix * v;
	v3 = v4.xyz / v4.w;

	// fully-transformed coordinates
	gl_Position = gl_ProjectionMatrix * v4;
}