﻿// $Id: GaussianBlur.frag 528 2011-04-18 17:51:13Z cr333 $
#version 120
#extension GL_EXT_gpu_shader4 : enable

// input tone images
uniform sampler2D input;

// standard deviation of the Gaussian blur
uniform float sigma;

// Applies a Gaussian blur to the RGB channels of the input image,
// while leaving the alpha channel untouched.
void main(void)
{
	vec2 texelSize = 1.0 / textureSize2D(input, 0).xy;

	// colour of central pixel
	vec4 pix = texture2D(input, gl_FragCoord.yx * texelSize);
	
	// homogeneous accumulator (i.e. acc.z is the sum of weights)
	vec4 acc = vec4(pix.rgb, 1);
	
	// precompute common term
	float ss = -0.5f / (sigma * sigma);
	
	// set kernel size to two sigma
	int radius = int(ceil(2.0 * sigma));
	
	// apply horizontal blur while transposing the image
	// (this allows the same shader to be applied twice for a 2D Gaussian blur)
	for(int i = 1; i <= radius; i++)
	{
		// the two pixels at distance i from the centre share the same weight
		vec3 pix1 = texture2D(input, (gl_FragCoord.yx + vec2(-i, 0)) * texelSize).rgb;
		vec3 pix2 = texture2D(input, (gl_FragCoord.yx + vec2( i, 0)) * texelSize).rgb;
		acc += exp(i * i * ss) * vec4(pix1 + pix2, 2);
	}

	gl_FragColor = vec4(acc.rgb / acc.w, pix.a);
	//gl_FragColor = vec4(acc.rgb, pix.a);
}