﻿// $Id: ParameterAttribute.cs 101 2009-09-24 17:24:58Z cr333 $
using System;

namespace VideoLib.Parameters
{
    /// <summary>
    /// Marks a property that can be found and changed at runtime such as a parameter of an algorithm.
    /// </summary>
    /// <remarks>The property targetted must support 'set'.</remarks>
    [AttributeUsage(AttributeTargets.Property, AllowMultiple = false)]
    public class ParameterAttribute : Attribute
    {
        private string friendlyName;
        private string description;
        private string groupId;

        private object minVal;
        private object maxVal;
        private object defaultVal;

        /// <summary>
        /// Initializes a new instance of the <see cref="ParameterAttribute"/> class.
        /// </summary>
        public ParameterAttribute()
        {
            friendlyName = null;
            description = "";
            groupId = "";

            minVal = null;
            maxVal = null;
            defaultVal = 0;
        }

        /// <summary>
        /// Gets or sets the friendly name that can be used as an identifier for the parameter.
        /// </summary>
        public string FriendlyName
        {
            get { return friendlyName; }
            set { friendlyName = value; }
        }

        /// <summary>
        /// Gets or sets a more detailed description or explanation of the parameter.
        /// </summary>
        public string Description
        {
            get { return description; }
            set { description = value; }
        }

        /// <summary>
        /// Gets or sets a 'group name', enabling parameters from a single object to be grouped under sub-headings, if required.
        /// </summary>
        public string GroupName
        {
            get { return groupId; }
            set { groupId = value; }
        }

        /// <summary>
        /// Gets or sets a minimum allowed value for the parameter.
        /// </summary>
        public object Minimum
        {
            get { return minVal; }
            set { minVal = value; }
        }

        /// <summary>
        /// Gets or sets a maximum allowed value for the parameter.
        /// </summary>
        public object Maximum
        {
            get { return maxVal; }
            set { maxVal = value; }
        }

        /// <summary>
        /// Gets or sets a default value for the parameter.
        /// </summary>
        public object Default
        {
            get { return defaultVal; }
            set { defaultVal = value; }
        }
    }
}
