﻿// $Id: SchedulerHelper.cs 293 2010-08-04 16:26:33Z cr333 $
// Based on a Part II project by Rubin Xu (2010).
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Threading;

namespace VideoLib.Scheduling
{
	static class SchedulerHelper
	{
		[DllImport("kernel32.dll")]
		static extern IntPtr GetCurrentThread();

		[DllImport("kernel32.dll")]
		static extern IntPtr SetThreadAffinityMask(IntPtr hThread, IntPtr dwThreadAffinityMask);

		/// <summary>Sets the current thread's processor affinity.</summary>
		/// <param name="coreId">Logical processor ID to pin the thread to.</param>
		/// <remarks>If the coreId exceeds the processor count, then the thread is not pinned.</remarks>
		public static void PinCurrentThreadToCore(int coreId)
		{
			// By default, threads can run on any logical processor
			// included in the current process' processor affinity.
			IntPtr affinity = Process.GetCurrentProcess().ProcessorAffinity;

			// We can only pin a thread to a logical processor, if the specified
			// core ID is valid according to the process' processor affinity.
			// This means we cannot pin a thread to a processor that the current
			// process cannot run on.
			long coreMask = 1L << coreId;
			if (coreId >= 0 && (affinity.ToInt64() & coreMask) > 0)
				affinity = new IntPtr(1 << coreId);

			// Some CLR implementations (such as SQL Server 2005 apparently) may move tasks
			// between physical threads, so we tell the CLR not to do that (just in case).
			Thread.BeginThreadAffinity();

			if (SetThreadAffinityMask(GetCurrentThread(), affinity) == IntPtr.Zero)
				throw new Win32Exception(Marshal.GetLastWin32Error());
		}

		public static void UnpinCurrentThread()
		{
			// reset thread affinity to the process' affinity
			SetThreadAffinityMask(GetCurrentThread(), Process.GetCurrentProcess().ProcessorAffinity);
			Thread.EndThreadAffinity();
		}
	}
}
