﻿// $Id: StatusWorkQueue.cs 440 2010-10-18 11:49:09Z cr333 $
// Partially based on a Part II project by Rubin Xu (2010).
using System.Collections.Generic;
using System.Threading;
using VideoLib.Frames;

namespace VideoLib.Scheduling
{
	/// <summary>Implements a work queue just for status updates and transporting frames.</summary>
	/// <remarks>Should be applied before <see cref="SharedWorkQueue"/>.</remarks>
	public class StatusWorkQueue : WorkQueue
	{
		/// <summary>Starts all worker threads associated with this work queue.</summary>
		/// <returns>Collection of all started worker threads.</returns>
		public override IEnumerable<Thread> StartWorkers()
		{
			StartWorker(0, "StatusWorker");
			return workers;
		}

		/// <summary>Handles new frame events.</summary>
		/// <param name="sender">The sender.</param>
		/// <param name="e">The <see cref="VideoLib.Frames.NewFrameEventArgs"/> instance containing the event data.</param>
		/// <returns>True, if and only if the event was handled.</returns>
		public override bool HandleNewFrameEvent(object sender, NewFrameEventArgs e)
		{
			return HandleNewFrameEvent(sender, e, Topology, Ready);
		}

		/// <summary>Handles changes of pin status.</summary>
		/// <param name="sender">The sender.</param>
		/// <param name="e">The <see cref="VideoLib.PinStatusChangedEventArgs"/> instance containing the event data.</param>
		/// <returns>True, if and only if the event was handled.</returns>
		public override bool HandlePinStatusChangedEvent(object sender, PinStatusChangedEventArgs e)
		{
			return HandlePinStatusChangedEvent(sender, e, Topology, Ready);
		}
	}
}